<?php

namespace Drupal\spamaway\Plugin\WebformHandler;

use Drupal\Core\Form\FormStateInterface;
use Drupal\webform\Plugin\WebformHandlerBase;
use Drupal\webform\WebformSubmissionInterface;

/**
 * Checks Anti SPAM of a submission.
 *
 * Checks if the current submission has been done before based on these criteria:
 * - within a specific period of time (configuratble)
 * - similar data has been posted for specific fields (based on similar_text percentage threshold)
 * - x amount of similar posts have been made.
 *
 * @WebformHandler(
 *   id = "spamaway_anti_spam_forms",
 *   label = @Translation("SpamAway - Anti spam handler"),
 *   category = @Translation("Anti-SPAM"),
 *   description = @Translation("SpamAway - Anti Spam based on repeated (similar) submissions."),
 *   cardinality = \Drupal\webform\Plugin\WebformHandlerInterface::CARDINALITY_SINGLE,
 *   results = \Drupal\webform\Plugin\WebformHandlerInterface::RESULTS_PROCESSED,
 * )
 */
class AntiSpamHandler extends WebformHandlerBase {

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {

    $form['spamaway_anti_spam_field_names'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Field names'),
      '#description' => $this->t('A comma seperated list of field names to take into consideration for similarity'),
      '#default_value' => $this->configuration['spamaway_anti_spam_field_names'] ?? 'message',
    ];

    $form['spamaway_anti_spam_threshold_percentage'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Threshold percentage'),
      '#suffix' => '%',
      '#description' => $this->t('A comma seperated list of threshold percentages for each field name (or one single value used for all field names)'),
      '#default_value' => $this->configuration['spamaway_anti_spam_threshold_percentage'] ?? 80,
    ];

    $form['spamaway_anti_spam_period'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Period of time'),
      '#description' => $this->t('The period for within similar submissions must have been submitted in seconds. Use 0(default) to disable this condition.'),
      '#suffix' => 'seconds',
      '#default_value' => $this->configuration['spamaway_anti_spam_period'] ?? 0,
    ];

    $form['spamaway_anti_spam_allowed_count'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Allowed count'),
      '#description' => $this->t('The number of similar submissions allowed before we consider it spam.'),
      '#default_value' => $this->configuration['spamaway_anti_spam_allowed_count'] ?? 5,
    ];

    $form['spamaway_anti_spam_ip_period'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Period of time for IP'),
      '#description' => $this->t('The period for within submissions with the same IP address are considered spam in seconds. Defaults to 10 minutes'),
      '#suffix' => 'seconds',
      '#default_value' => $this->configuration['spamaway_anti_spam_ip_period'] ?? 36000,
    ];

    $form['spamaway_anti_spam_allowed_ip_count'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Allowed IP count'),
      '#description' => $this->t('The number of similar submissions allowed by the same IP address before we consider it spam. Defaults to 4'),
      '#default_value' => $this->configuration['spamaway_anti_spam_allowed_ip_count'] ?? 4,
    ];

    $form['spamaway_anti_spam_logging'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Enable logging'),
      '#default_value' => $this->configuration['spamaway_anti_spam_logging'] ?? 0,
    ];

    $form['spamaway_query_limit'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Query limit'),
      '#description' => $this->t('Cannot be higher than 200'),
      '#default_value' => $this->configuration['spamaway_query_limit'] ?? 200,
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    $this->configuration['spamaway_anti_spam_field_names'] = $form_state->getValue('spamaway_anti_spam_field_names');
    $this->configuration['spamaway_anti_spam_period'] = $form_state->getValue('spamaway_anti_spam_period');
    $this->configuration['spamaway_anti_spam_allowed_count'] = $form_state->getValue('spamaway_anti_spam_allowed_count');
    $this->configuration['spamaway_anti_spam_threshold_percentage'] = $form_state->getValue('spamaway_anti_spam_threshold_percentage');
    $this->configuration['spamaway_anti_spam_ip_period'] = $form_state->getValue('spamaway_anti_spam_ip_period');
    $this->configuration['spamaway_anti_spam_allowed_ip_count'] = $form_state->getValue('spamaway_anti_spam_allowed_ip_count');
    $this->configuration['spamaway_anti_spam_logging'] = $form_state->getValue('spamaway_anti_spam_logging');
    $this->configuration['spamaway_query_limit'] = $form_state->getValue('spamaway_query_limit');
  }

  /**
   * Validate webform submission webform .
   *
   * @param array $form
   *   An associative array containing the structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   * @param \Drupal\webform\WebformSubmissionInterface $webform_submission
   *   A webform submission.
   */
  public function validateForm(array &$form, FormStateInterface $form_state, WebformSubmissionInterface $webform_submission) {

    $connection = \Drupal::database();
    $logging = $this->configuration['spamaway_anti_spam_logging'] ?? 0;

    // Set hard limit
    if ($this->configuration['spamaway_query_limit'] > 200) {
      $this->configuration['spamaway_query_limit'] = 200;
    }

    ////////////////////////////////////////////////////////////////////////////
    // Make sure the last x seconds no submission was done by the same IP
    // address. Otherwise consider it spam.
    $spamaway_anti_spam_allowed_ip_count = $this->configuration['spamaway_anti_spam_allowed_ip_count'];
    $spamaway_anti_spam_ip_period = $this->configuration['spamaway_anti_spam_ip_period'];
    $submit_remote_addr = $webform_submission->getRemoteAddr();

    $query_ip = $connection->select('webform_submission', 'w');
    $query_ip->condition('remote_addr', $submit_remote_addr);
    $query_ip->where('created > UNIX_TIMESTAMP() - :limit', [':limit' => $spamaway_anti_spam_ip_period]);
    $count_ip = $query_ip->countQuery()->execute()->fetchField();
    if ($count_ip > $spamaway_anti_spam_allowed_ip_count) {
      // We had (a) very similar submission(s) before so we ignore the new submission.
      $form_state->setErrorByName('', $this->t('Spam detected. Please contact the site administrator if the issue persists.'));

      if ($logging) {
        $this->getLogger('spamaway_spam')->debug('IP spam ' . $submit_remote_addr . ' within ' . $spamaway_anti_spam_ip_period . ' (count ' . $count_ip .'/' . $spamaway_anti_spam_allowed_ip_count . ')');
      }
    }

    ////////////////////////////////////////////////////////////////////////////
    // Check if the submitted message is similar to previous submitted messages

    // Lookup X last submissions.
    $query = $connection->select('webform_submission', 's');
    $query->addField('d', 'value', 'value');
    $query->addField('d', 'name', 'name');
    $query->leftJoin('webform_submission_data', 'd', 'd.sid = s.sid');
    $query->range(0, $this->configuration['spamaway_query_limit']);
    $query->orderBy('s.created', 'DESC');

    $period = time() - $this->configuration['spamaway_anti_spam_period'];
    if ($this->configuration['spamaway_anti_spam_period'] && $period > 0) {
      $query->condition('s.created', $period, '>');
    }

    // Get the submitted values and while we are at it,
    // also create the condition statements.
    $field_conditions = $query->orConditionGroup();
    $form_values = [];
    $names = explode(',', $this->configuration['spamaway_anti_spam_field_names']);
    $thresholds = explode(',', $this->configuration['spamaway_anti_spam_threshold_percentage']);

    foreach ($names as $index => $name) {
      $form_values[$name] = $form_state->getValue($name);
      $field_conditions->condition('d.name', $name);

      if (count($thresholds) === 1) {
        $thresholds[$name] = $thresholds[0];
      }
      else {
        $thresholds[$name] = $thresholds[$index];
      }
    }

    $query->condition($field_conditions);
    $query->condition('d.webform_id', $webform_submission->getWebform()->id());
    $rows = $query->execute()->fetchAll();

    $precentage = 0;
    $similar_count = 0;

    foreach ($rows as $row) {

      // Calculate spaminess.
      $matching = similar_text($form_values[$row->name], $row->value, $precentage);

      // Check if spaminess is higher than config value.
      if ($matching && $precentage > $thresholds[$row->name]) {

        $similar_count++;

        if ($similar_count < $this->configuration['spamaway_anti_spam_allowed_count']) {
          continue;
        }

        // We had (a) very similar submission(s) before so we ignore the new submission.
        $form_state->setErrorByName('', $this->t('Spam detected. Please contact the site administrator if the issue persists.'));

        if ($logging) {
          $this->getLogger('spamaway_spam')->debug('Similar spam ' . $precentage . ' (count ' . $similar_count . ')');
        }

        break;
      }
    }
  }

}
